@@ -22,8 +22,8 @@
 #include "luatexfont.h"
 
 static const char _svn_version[] =
-    "$Id: writefont.c 3011 2009-08-17 14:15:46Z taco $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/font/writefont.c $";
+    "$Id: writefont.c 3073 2009-10-19 11:26:44Z taco $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/font/writefont.c $";
 
 void write_cid_fontdictionary(PDF pdf, fo_entry * fo, internalfontnumber f);
 void create_cid_fontdictionary(PDF pdf,
@@ -203,7 +203,7 @@ static void write_fontmetrics(PDF pdf, fd_entry * fd)
 
 static void preset_fontname(fo_entry * fo, internalfontnumber f)
 {
-    if (font_fullname(f)!= NULL)
+    if (font_fullname(f) != NULL)
         fo->fd->fontname = xstrdup(font_fullname(f));
     else if (fo->fm->ps_name != NULL)
         fo->fd->fontname = xstrdup(fo->fm->ps_name);    /* just fallback */
@@ -533,7 +533,7 @@ static void write_fontdescriptor(PDF pdf, fd_entry * fd)
     int fd_flags;
     assert(fd != NULL && fd->fm != NULL);
 
-    if (is_fontfile(fd->fm))
+    if (is_fontfile(fd->fm) && is_included(fd->fm))
         write_fontfile(pdf, fd);        /* this will set fd->ff_found if font file is found */
     if (fd->fn_objnum != 0)
         write_fontname_object(pdf, fd);
@@ -792,9 +792,9 @@ void do_pdf_font(PDF pdf, integer font_objnum, internalfontnumber f)
         fm->tfm_name = font_name(f);    /* or whatever, not a real tfm */
         fm->ff_name = font_filename(f); /* the actual file */
         if (font_psname(f) != NULL)
-            fm->ps_name = font_psname(f); /* the true name */
+            fm->ps_name = font_psname(f);       /* the true name */
         else
-            fm->ps_name = font_fullname(f); /* the true name */
+            fm->ps_name = font_fullname(f);     /* the true name */
         if (fm->ff_name
             && strlen(fm->ff_name) >= 6
             && strstr(fm->ff_name,
