@@ -54,8 +54,8 @@
 #include "epdf.h"
 
 static const char _svn_version[] =
-    "$Id: pdftoepdf.cc 2997 2009-08-14 14:17:07Z oneiros $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/image/pdftoepdf.cc $";
+    "$Id: pdftoepdf.cc 3029 2009-08-19 13:19:30Z oneiros $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/image/pdftoepdf.cc $";
 
 #define one_hundred_bp  6578176 /* 7227 * 65536 / 72 */
 
@@ -141,8 +141,7 @@ struct PdfDocument {
 
 /* AVL sort PdfDocument into PdfDocumentTree by file_path */
 
-static int CompPdfDocument(const void *pa, const void *pb, void *p
-                           __attribute__ ((unused)))
+static int CompPdfDocument(const void *pa, const void *pb, void * /*p */ )
 {
     return strcmp(((const PdfDocument *) pa)->file_path,
                   ((const PdfDocument *) pb)->file_path);
@@ -1037,7 +1036,7 @@ static void deletePdfDocument(PdfDocument * pdf_doc)
     delete pdf_doc;
 }
 
-static void destroyPdfDocument(void *pa, void *pb __attribute__ ((unused)))
+static void destroyPdfDocument(void *pa, void * /*pb */ )
 {
     PdfDocument *pdf_doc = (PdfDocument *) pa;
     deletePdfDocument(pdf_doc);
