@@ -23,7 +23,7 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: luainit.c 2899 2009-07-21 23:03:53Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/lua/luainit.c $";
+    "$Id: luainit.c 3078 2009-10-25 22:52:31Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/lua/luainit.c $";
 
 /* TH: TODO
  *
@@ -469,15 +469,18 @@ void fix_dumpname(void)
     }
 }
 
+int tex_table_id;
+int pdf_table_id;
+int token_table_id;
+int node_table_id;
+
+extern void init_tex_table(lua_State * L);
+
 void lua_initialize(int ac, char **av)
 {
 
     char *given_file = NULL;
     int kpse_init;
-    int tex_table_id;
-    int pdf_table_id;
-    int token_table_id;
-    int node_table_id;
     /* Save to pass along to topenin.  */
     argc = ac;
     argv = av;
@@ -541,6 +544,8 @@ void lua_initialize(int ac, char **av)
             fprintf(stdout, "%s\n", lua_tostring(Luas, -1));
             exit(1);
         }
+        /* */
+        init_tex_table(Luas);
         if (lua_pcall(Luas, 0, 0, 0)) {
             fprintf(stdout, "%s\n", lua_tostring(Luas, -1));
             exit(1);
