@@ -11,12 +11,16 @@
    Makefile rule.  */
 #include "luatex.h"
 
+static const char __svn_version[] =
+    "$Id: luatex.c 3081 2009-10-25 22:57:52Z oneiros $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/luatex.c $";
+
 #define TeX
 
-int luatex_version = 43;        /* \.{\\luatexversion}  */
+int luatex_version = 44;        /* \.{\\luatexversion}  */
 int luatex_revision = '0';      /* \.{\\luatexrevision}  */
 int luatex_date_info = -extra_version_info;     /* the compile date is negated */
-char *luatex_version_string = "beta-0.43.0";
+char *luatex_version_string = "beta-0.44.0";
 char *engine_name = "luatex";   /* the name of this engine */
 
 
