@@ -18,8 +18,8 @@
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
 static const char __svn_version[] =
-    "$Id: pdfgen.c 3025 2009-08-18 22:42:49Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/pdf/pdfgen.c $";
+    "$Id: pdfgen.c 3077 2009-10-25 22:51:23Z oneiros $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdfgen.c $";
 
 #include "ptexlib.h"
 #include <ctype.h>
@@ -59,7 +59,7 @@ PDF init_pdf_struct(PDF pdf)
     memset(pdf, 0, sizeof(pdf_output_file));
 
     pdf->o_mode = OMODE_NONE;   /* will be set by fix_o_mode() */
-    pdf->o_status = ST_INITIAL;
+    pdf->o_state = ST_INITIAL;
 
     pdf->os_obj = xmalloc(pdf_os_max_objs * sizeof(os_obj_data));
     pdf->os_buf_size = inf_pdf_os_buf_size;
@@ -942,46 +942,92 @@ static void init_pdf_outputparameters(PDF pdf)
 
 /* Checks that we have a name for the generated PDF file and that it's open. */
 
-static void ensure_pdf_open(PDF pdf)
+static void ensure_output_file_open(PDF pdf, char *s)
 {
     if (pdf->file_name != NULL)
         return;
     if (job_name == 0)
         open_log_file();
-    pack_job_name(".pdf");
-    if (pdf->draftmode == 0) {
+    pack_job_name(s);
+    if (pdf->draftmode == 0 || pdf->o_mode == OMODE_DVI) {
         while (!lua_b_open_out(pdf->file))
-            prompt_file_name("file name for output", ".pdf");
+            prompt_file_name("file name for output", s);
     }
     pdf->file = name_file_pointer;      /* hm ? */
     pdf->file_name = xstrdup(makecstring(make_name_string()));
 }
 
-void ensure_pdf_header_written(PDF pdf)
+static void ensure_pdf_header_written(PDF pdf)
 {
-    static boolean header_written = false;      /* kludge, should be in pdf */
-    if (!header_written) {
-        assert(pdf->o_mode == OMODE_PDF);
-        ensure_pdf_open(pdf);
-        /* Initialize variables for \.{PDF} output */
-        fix_pdf_minorversion(pdf);
-        init_pdf_outputparameters(pdf);
-        /* Write \.{PDF} header */
-        pdf_printf(pdf, "%%PDF-1.%d\n", pdf->minor_version);
-        pdf_out(pdf, '%');
-        pdf_out(pdf, 'P' + 128);
-        pdf_out(pdf, 'T' + 128);
-        pdf_out(pdf, 'E' + 128);
-        pdf_out(pdf, 'X' + 128);
-        pdf_print_nl(pdf);
-        header_written = true;
+    assert(pdf->o_state == ST_FILE_OPEN);
+    assert(pdf->o_mode == OMODE_PDF);
+    /* Initialize variables for \.{PDF} output */
+    fix_pdf_minorversion(pdf);
+    init_pdf_outputparameters(pdf);
+    /* Write \.{PDF} header */
+    pdf_printf(pdf, "%%PDF-1.%d\n", pdf->minor_version);
+    pdf_out(pdf, '%');
+    pdf_out(pdf, 'P' + 128);
+    pdf_out(pdf, 'T' + 128);
+    pdf_out(pdf, 'E' + 128);
+    pdf_out(pdf, 'X' + 128);
+    pdf_print_nl(pdf);
+}
+
+void ensure_output_state(PDF pdf, output_state s)
+{
+    if (pdf->o_state < s) {
+        if (s > ST_INITIAL)
+            ensure_output_state(pdf, s - 1);
+        switch (s - 1) {
+        case ST_INITIAL:
+            fix_o_mode(pdf);
+            break;
+        case ST_OMODE_FIX:
+            switch (pdf->o_mode) {
+            case OMODE_DVI:
+                ensure_output_file_open(pdf, ".dvi");
+                break;
+            case OMODE_PDF:
+                ensure_output_file_open(pdf, ".pdf");
+                break;
+            case OMODE_LUA:
+                break;
+            default:
+                assert(0);
+            }
+            break;
+        case ST_FILE_OPEN:
+            switch (pdf->o_mode) {
+            case OMODE_DVI:
+                ensure_dvi_header_written(pdf);
+                break;
+            case OMODE_PDF:
+                ensure_pdf_header_written(pdf);
+                break;
+            case OMODE_LUA:
+                break;
+            default:
+                assert(0);
+            }
+            break;
+        case ST_HEADER_WRITTEN:
+            break;
+        case ST_FILE_CLOSED:
+            break;
+        default:
+            assert(0);
+        }
+        pdf->o_state++;
     }
 }
 
+/**********************************************************************/
+
 /* begin a PDF dictionary object */
 void pdf_begin_dict(PDF pdf, integer i, integer pdf_os_level)
 {
-    ensure_pdf_header_written(pdf);
+    ensure_output_state(pdf, ST_HEADER_WRITTEN);
     pdf_os_prepare_obj(pdf, i, pdf_os_level);
     if (!pdf->os_mode) {
         pdf_printf(pdf, "%d 0 obj <<\n", (int) i);
@@ -1073,7 +1119,7 @@ void pdf_os_write_objstream(PDF pdf)
 /* begin a PDF object */
 void pdf_begin_obj(PDF pdf, integer i, integer pdf_os_level)
 {
-    ensure_pdf_header_written(pdf);
+    ensure_output_state(pdf, ST_HEADER_WRITTEN);
     pdf_os_prepare_obj(pdf, i, pdf_os_level);
     if (!pdf->os_mode) {
         pdf_printf(pdf, "%d 0 obj\n", (int) i);
@@ -1535,7 +1581,7 @@ void pdf_warning(char *t, char *p, boolean prepend_nl, boolean append_nl)
 /**********************************************************************/
 /* Use check_o_mode() in the backend-specific "Implement..." chunks */
 
-void check_o_mode(PDF pdf, char *s, int o_modes, boolean strict)
+void check_o_mode(PDF pdf, char *s, int o_mode_bitpattern, boolean strict)
 {
 
     char warn_string[100];
@@ -1557,7 +1603,7 @@ void check_o_mode(PDF pdf, char *s, int o_modes, boolean strict)
             o_mode = OMODE_DVI;
     } else
         o_mode = pdf->o_mode;
-    if ((o_mode & o_modes) == 0) {      /* warning or error */
+    if (!((1 << o_mode) & o_mode_bitpattern)) { /* warning or error */
         switch (o_mode) {
         case OMODE_DVI:
             m = "DVI";
@@ -1572,28 +1618,13 @@ void check_o_mode(PDF pdf, char *s, int o_modes, boolean strict)
             assert(0);
         }
         snprintf(warn_string, 99, "not allowed in %s mode (\\pdfpoutput = %d)",
-                 m, pdf_output);
+                 m, (int) pdf_output);
         if (strict)
             pdf_error(s, warn_string);
         else
             pdf_warning(s, warn_string, true, true);
-    } else {
-        if (strict) {
-            fix_o_mode(pdf);
-            switch (o_mode) {
-            case OMODE_DVI:
-                ensure_dvi_header_written(pdf);
-                break;
-            case OMODE_PDF:
-                ensure_pdf_header_written(pdf);
-                break;
-            case OMODE_LUA:
-                break;
-            default:
-                assert(0);
-            }
-        }
-    }
+    } else if (strict)
+        ensure_output_state(pdf, ST_HEADER_WRITTEN);
 }
 
 /**********************************************************************/
@@ -1650,7 +1681,7 @@ void set_job_id(PDF pdf, int year, int month, int day, int time)
 void pdf_begin_page(PDF pdf, boolean shipping_page)
 {
     scaled form_margin = one_bp;
-    ensure_pdf_header_written(pdf);
+    ensure_output_state(pdf, ST_HEADER_WRITTEN);
     init_pdf_pagecalculations(pdf);
 
     if (pdf->resources == NULL)
@@ -2224,9 +2255,9 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
                 build_free_object_list(pdf);
                 pdf_printf(pdf, "/Type /XRef\n");
                 pdf_printf(pdf, "/Index [0 ");
-                pdf_print_int(pdf, pdf->obj_ptr);
+                pdf_print_int(pdf, pdf->obj_ptr + 1);
                 pdf_printf(pdf, "]\n");
-                pdf_int_entry_ln(pdf, "Size", pdf->obj_ptr);
+                pdf_int_entry_ln(pdf, "Size", pdf->obj_ptr + 1);
                 pdf_printf(pdf, "/W [1 ");
                 pdf_print_int(pdf, xref_offset_width);
                 pdf_printf(pdf, " 1]\n");
@@ -2258,6 +2289,7 @@ void finish_pdf_file(PDF pdf, integer luatex_version,
                     }
                 }
                 pdf_end_stream(pdf);
+                /* TODO: generate a debug version of the crossref */
 
                 pdf_flush(pdf);
             } else {
@@ -2363,7 +2395,7 @@ void scan_pdfcatalog(PDF pdf)
         if (pdf_catalog_openaction != 0) {
             pdf_error("ext1", "duplicate of openaction");
         } else {
-            check_o_mode(pdf, "\\pdfcatalog", OMODE_PDF, true);
+            check_o_mode(pdf, "\\pdfcatalog", 1 << OMODE_PDF, true);
             p = scan_action(pdf);
             pdf_new_obj(pdf, obj_type_others, 0, 1);
             pdf_catalog_openaction = pdf->obj_ptr;
