@@ -17,144 +17,105 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-#include "ptexlib.h"
-
 static const char __svn_version[] =
-    "$Id: pdflistout.c 3022 2009-08-18 17:52:15Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/pdf/pdflistout.c $";
+    "$Id: pdflistout.c 3058 2009-10-14 06:51:27Z taco $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/pdf/pdflistout.c $";
 
-/***********************************************************************/
-
-static void late_lua(PDF pdf, halfword p)
-{
-    (void) pdf;
-    expand_macros_in_tokenlist(p);      /* sets def_ref */
-    luacall(def_ref, late_lua_name(p));
-    flush_list(def_ref);
-}
+#include "ptexlib.h"
 
 pos_info_structure pos_info;    /* to be accessed from Lua */
 
-/***********************************************************************/
-/* TODO: remove these: */
-
-#define push 141
-
-#define dvi_out(A) do {                 \
-    dvi_buf[dvi_ptr++]=A;               \
-    if (dvi_ptr==dvi_limit) dvi_swap(); \
-  } while (0)
-
-/***********************************************************************/
-
-backend_node_function *backend_out = NULL;
-backend_whatsit_function *backend_out_whatsit = NULL;
+static backend_struct *backend = NULL;
+backend_function *backend_out, *backend_out_whatsit;
 
 static void missing_backend_function(PDF pdf, halfword p)
 {
-    char *b = NULL, *n, *s;
+    char *b, *n, *s;
     char backend_string[15];
     char err_string[60];
+    assert(backend != NULL);
     if (type(p) == whatsit_node)
         s = "whatsit";
     else
         s = "node";
     n = get_node_name(type(p), subtype(p));
-    switch (pdf->o_mode) {
-    case OMODE_DVI:
-        b = "DVI";
-        break;
-    case OMODE_PDF:
-        b = "PDF";
-        break;
-    case OMODE_LUA:
-        b = "Lua";
-        break;
-    default:
-        assert(0);
-    }
-    snprintf(backend_string, 14, "%s back-end", b);
+    b = backend[pdf->o_mode].name;
+    snprintf(backend_string, strlen(b) + 10, "%s back-end", b);
     snprintf(err_string, 59, "no output function for \"%s\" %s", n, s);
     pdf_error(backend_string, err_string);
 }
 
-static backend_node_function *new_backend_out()
+static void init_none_backend_functions()
 {
-    assert(backend_out == NULL);
-    return xmalloc((MAX_NODE_TYPE + 1) * sizeof(backend_node_function));
+    backend_struct *p = &backend[OMODE_NONE];
+    p->name = strdup("(None)");
 }
 
-static backend_node_function *new_backend_out_whatsit()
+static void init_pdf_backend_functions()
 {
-    assert(backend_out_whatsit == NULL);
-    return xmalloc((MAX_WHATSIT_TYPE + 1) * sizeof(backend_whatsit_function));
+    backend_struct *p = &backend[OMODE_PDF];
+    p->name = strdup("PDF");
+    p->node_fu[rule_node] = &pdf_place_rule;    /* 2 */
+    p->node_fu[glyph_node] = &pdf_place_glyph;  /* 37 */
+    p->whatsit_fu[special_node] = &pdf_special; /* 3 */
+    p->whatsit_fu[pdf_literal_node] = &pdf_out_literal; /* 8 */
+    p->whatsit_fu[pdf_refobj_node] = &pdf_ref_obj;      /* 10 */
+    p->whatsit_fu[pdf_refxform_node] = &pdf_place_form; /* 12 */
+    p->whatsit_fu[pdf_refximage_node] = &pdf_place_image;       /* 14 */
+    p->whatsit_fu[pdf_annot_node] = &do_annot;  /* 15 */
+    p->whatsit_fu[pdf_start_link_node] = &do_link;      /* 16 */
+    p->whatsit_fu[pdf_end_link_node] = &end_link;       /* 17 */
+    p->whatsit_fu[pdf_dest_node] = &do_dest;    /* 19 */
+    p->whatsit_fu[pdf_thread_node] = &do_thread;        /* 20 */
+    p->whatsit_fu[pdf_end_thread_node] = &end_thread;   /* 22 */
+    p->whatsit_fu[late_lua_node] = &late_lua;   /* 35 */
+    p->whatsit_fu[pdf_colorstack_node] = &pdf_out_colorstack;   /* 39 */
+    p->whatsit_fu[pdf_setmatrix_node] = &pdf_out_setmatrix;     /* 40 */
+    p->whatsit_fu[pdf_save_node] = &pdf_out_save;       /* 41 */
+    p->whatsit_fu[pdf_restore_node] = &pdf_out_restore; /* 42 */
 }
 
-static void init_pdf_backend_functionpointers(PDF pdf __attribute__ ((unused)))
+static void init_dvi_backend_functions()
 {
-    backend_out[rule_node] = &pdf_place_rule;   /* 2 */
-    backend_out[glyph_node] = &pdf_place_glyph; /* 37 */
-    /* ...these are all (?) */
-    backend_out_whatsit[special_node] = &pdf_special;   /* 3 */
-    backend_out_whatsit[pdf_literal_node] = &pdf_out_literal;   /* 8 */
-    backend_out_whatsit[pdf_refobj_node] = &pdf_ref_obj;        /* 10 */
-    backend_out_whatsit[pdf_refxform_node] = &pdf_place_form;   /* 12 */
-    backend_out_whatsit[pdf_refximage_node] = &pdf_place_image; /* 14 */
-    backend_out_whatsit[pdf_annot_node] = &do_annot;    /* 15 */
-    backend_out_whatsit[pdf_start_link_node] = &do_link;        /* 16 */
-    backend_out_whatsit[pdf_end_link_node] = &end_link; /* 17 */
-    backend_out_whatsit[pdf_dest_node] = &do_dest;      /* 19 */
-    backend_out_whatsit[pdf_thread_node] = &do_thread;  /* 20 */
-    backend_out_whatsit[pdf_end_thread_node] = &end_thread;     /* 22 */
-    backend_out_whatsit[late_lua_node] = &late_lua;     /* 35 */
-    backend_out_whatsit[pdf_colorstack_node] = &pdf_out_colorstack;     /* 39 */
-    backend_out_whatsit[pdf_setmatrix_node] = &pdf_out_setmatrix;       /* 40 */
-    backend_out_whatsit[pdf_save_node] = &pdf_out_save; /* 41 */
-    backend_out_whatsit[pdf_restore_node] = &pdf_out_restore;   /* 42 */
+    backend_struct *p = &backend[OMODE_DVI];
+    p->name = strdup("DVI");
+    p->node_fu[rule_node] = &dvi_place_rule;    /* 2 */
+    p->node_fu[glyph_node] = &dvi_place_glyph;  /* 37 */
+    p->whatsit_fu[special_node] = &dvi_special; /* 3 */
+    p->whatsit_fu[late_lua_node] = &late_lua;   /* 35 */
 }
 
-static void init_dvi_backend_functionpointers(PDF pdf __attribute__ ((unused)))
+static void init_lua_backend_functions()
 {
-    backend_out[rule_node] = &dvi_place_rule;   /* 2 */
-    backend_out[glyph_node] = &dvi_place_glyph; /* 37 */
-    /* ...these are all (?) */
-    backend_out_whatsit[special_node] = &dvi_special;   /* 3 */
+    backend_struct *p = &backend[OMODE_LUA];
+    p->name = strdup("Lua");
+    p->node_fu[rule_node] = &lua_place_rule;    /* 2 */
+    p->node_fu[glyph_node] = &lua_place_glyph;  /* 37 */
+    p->whatsit_fu[late_lua_node] = &late_lua;   /* 35 */
 }
 
-static void init_lua_backend_functionpointers(PDF pdf __attribute__ ((unused)))
+void init_backend_functionpointers(output_mode o_mode)
 {
-    backend_out[rule_node] = &lua_place_rule;   /* 2 */
-    backend_out[glyph_node] = &lua_place_glyph; /* 37 */
-    /* ...these are all (?) */
-    backend_out_whatsit[late_lua_node] = &late_lua;     /* 35 */
-}
-
-void init_backend_functionpointers(PDF pdf)
-{
-    int i;
-    if (backend_out == NULL || backend_out_whatsit == NULL) {
-        backend_out = new_backend_out();
-        backend_out_whatsit = new_backend_out_whatsit();
-    }
-    for (i = 0; i < MAX_NODE_TYPE + 1; i++)
-        backend_out[i] = &missing_backend_function;
-    for (i = 0; i < MAX_WHATSIT_TYPE + 1; i++)
-        backend_out_whatsit[i] = &missing_backend_function;
-    switch (pdf->o_mode) {
-    case OMODE_NONE:           /* all node types give errors */
-        break;
-    case OMODE_DVI:
-        init_dvi_backend_functionpointers(pdf);
-        break;
-    case OMODE_PDF:
-        init_pdf_backend_functionpointers(pdf);
-        break;
-    case OMODE_LUA:
-        init_lua_backend_functionpointers(pdf);
-        break;
-    default:
-        assert(0);
+    int i, j;
+    if (backend == NULL) {
+        backend = xmalloc((MAX_OMODE + 1) * sizeof(backend_struct));
+        for (i = 0; i <= MAX_OMODE; i++) {
+            backend[i].node_fu =
+                xmalloc((MAX_NODE_TYPE + 1) * sizeof(backend_function));
+            backend[i].whatsit_fu =
+                xmalloc((MAX_WHATSIT_TYPE + 1) * sizeof(backend_function));
+            for (j = 0; j < MAX_NODE_TYPE + 1; j++)
+                backend[i].node_fu[j] = &missing_backend_function;
+            for (j = 0; j < MAX_WHATSIT_TYPE + 1; j++)
+                backend[i].whatsit_fu[j] = &missing_backend_function;
+        }
+        init_none_backend_functions();
+        init_dvi_backend_functions();
+        init_pdf_backend_functions();
+        init_lua_backend_functions();
     }
+    backend_out = backend[o_mode].node_fu;
+    backend_out_whatsit = backend[o_mode].whatsit_fu;
 }
 
 /***********************************************************************/
@@ -387,7 +348,7 @@ void hlist_out(PDF pdf, halfword this_box)
 
     if (pdf->o_mode == OMODE_DVI) {
         if (cur_s > 0) {        /* DVI! */
-            dvi_out(push);      /* DVI! */
+            dvi_push();         /* DVI! */
             save_dvi = dvi;     /* DVI! */
         }
         save_loc = dvi_offset + dvi_ptr /* DVI! */ ;
@@ -402,7 +363,7 @@ void hlist_out(PDF pdf, halfword this_box)
     }
 
     /* Start hlist {\sl Sync\TeX} information record */
-    if (synctexoption == 1)
+    if (int_par(synctex_code))
         synctex_hlist(this_box);
 
     while (p != null) {
@@ -424,7 +385,7 @@ void hlist_out(PDF pdf, halfword this_box)
                 p = vlink(p);
             } while (is_char_node(p));
             /* Record current point {\sl Sync\TeX} information */
-            if (synctexoption == 1)
+            if (int_par(synctex_code))
                 synctex_current();
         } else {
             /* Output the non-|char_node| |p| for |hlist_out|
@@ -490,7 +451,7 @@ void hlist_out(PDF pdf, halfword this_box)
                     basepoint.v = basepoint.v - shift_amount(p);        /* shift the box `up' */
                 if (list_ptr(p) == null) {
                     /* Record void list {\sl Sync\TeX} information */
-                    if (synctexoption == 1) {
+                    if (int_par(synctex_code)) {
                         if (type(p) == vlist_node)
                             synctex_void_vlist(p, this_box);
                         else
@@ -536,8 +497,7 @@ void hlist_out(PDF pdf, halfword this_box)
                 /* Output the whatsit node |p| in |hlist_out| */
                 switch (subtype(p)) {
                 case pdf_save_pos_node:
-                    pdf_last_x_pos = pdf->posstruct->pos.h;
-                    pdf_last_y_pos = pdf->posstruct->pos.v;
+                    pdf_last_pos = pdf->posstruct->pos;
                     pos_info.curpos = pdf->posstruct->pos;
                     pos_info.boxpos.pos = refpos->pos;
                     pos_info.boxpos.dir = localpos.dir;
@@ -553,8 +513,8 @@ void hlist_out(PDF pdf, halfword this_box)
                 case pdf_thread_node:  /* do_thread(pdf, p, this_box, cur); */
                     backend_out_whatsit[subtype(p)] (pdf, p, this_box, cur);
                     break;
-                case pdf_refxform_node:
-                case pdf_refximage_node:
+                case pdf_refxform_node:        /* pdf_place_form(pdf, pdf_xform_objnum(p)); */
+                case pdf_refximage_node:       /* pdf_place_image(pdf, pdf_ximage_idx(p)); */
                     /* Output a Form node in a hlist */
                     /* Output an Image node in a hlist */
                     switch (box_direction(localpos.dir)) {
@@ -587,10 +547,7 @@ void hlist_out(PDF pdf, halfword this_box)
                         pos_left(depth(p));
                         break;
                     }
-                    if (subtype(p) == pdf_refximage_node)       /* pdf_place_image(pdf, pdf_ximage_idx(p)); */
-                        backend_out_whatsit[pdf_refximage_node] (pdf, p);
-                    else        /* pdf_place_form(pdf, pdf_xform_objnum(p)); */
-                        backend_out_whatsit[pdf_refxform_node] (pdf, p);
+                    backend_out_whatsit[subtype(p)] (pdf, p);
                     cur.h += width(p);
                     break;
                 case dir_node:
@@ -787,13 +744,13 @@ void hlist_out(PDF pdf, halfword this_box)
                 break;
             case kern_node:
                 /* Record |kern_node| {\sl Sync\TeX} information */
-                if (synctexoption == 1)
+                if (int_par(synctex_code))
                     synctex_kern(p, this_box);
                 cur.h += width(p);
                 break;
             case math_node:
                 /* Record |math_node| {\sl Sync\TeX} information */
-                if (synctexoption == 1)
+                if (int_par(synctex_code))
                     synctex_math(p, this_box);
                 cur.h += surround(p);
                 break;
@@ -860,7 +817,7 @@ void hlist_out(PDF pdf, halfword this_box)
           MOVE_PAST:
             cur.h += rule.wd;
             /* Record horizontal |rule_node| or |glue_node| {\sl Sync\TeX} information */
-            if (synctexoption == 1) {
+            if (int_par(synctex_code)) {
                 synch_pos_with_cur(pdf->posstruct, refpos, cur);
                 synctex_horizontal_rule_or_glue(p, this_box);
             }
@@ -872,7 +829,7 @@ void hlist_out(PDF pdf, halfword this_box)
     }
 
     /* Finish hlist {\sl Sync\TeX} information record */
-    if (synctexoption == 1)
+    if (int_par(synctex_code))
         synctex_tsilh(this_box);
 
     if (pdf->o_mode == OMODE_DVI) {
@@ -930,14 +887,14 @@ void vlist_out(PDF pdf, halfword this_box)
 
     if (pdf->o_mode == OMODE_DVI) {
         if (cur_s > 0) {        /* DVI! */
-            dvi_out(push);      /* DVI! */
+            dvi_push();         /* DVI! */
             save_dvi = dvi;     /* DVI! */
         }
         save_loc = dvi_offset + dvi_ptr;        /* DVI! */
     }
 
     /* Start vlist {\sl Sync\TeX} information record */
-    if (synctexoption == 1)
+    if (int_par(synctex_code))
         synctex_vlist(this_box);
 
     /* Create thread for the current vbox if needed */
@@ -1000,7 +957,7 @@ void vlist_out(PDF pdf, halfword this_box)
                 if (list_ptr(p) == null) {
                     cur.v += effective_vertical;
                     /* Record void list {\sl Sync\TeX} information */
-                    if (synctexoption == 1) {
+                    if (int_par(synctex_code)) {
                         synch_pos_with_cur(pdf->posstruct, refpos, cur);
                         if (type(p) == vlist_node)
                             synctex_void_vlist(p, this_box);
@@ -1036,8 +993,7 @@ void vlist_out(PDF pdf, halfword this_box)
                 /* Output the whatsit node |p| in |vlist_out| */
                 switch (subtype(p)) {
                 case pdf_save_pos_node:
-                    pdf_last_x_pos = pdf->posstruct->pos.h;
-                    pdf_last_y_pos = pdf->posstruct->pos.v;
+                    pdf_last_pos = pdf->posstruct->pos;
                     pos_info.curpos = pdf->posstruct->pos;
                     pos_info.boxpos.pos = refpos->pos;
                     pos_info.boxpos.dir = localpos.dir;
@@ -1053,8 +1009,8 @@ void vlist_out(PDF pdf, halfword this_box)
                 case pdf_thread_node:  /* do_thread(pdf, p, this_box, cur); */
                     backend_out_whatsit[subtype(p)] (pdf, p, this_box, cur);
                     break;
-                case pdf_refxform_node:
-                case pdf_refximage_node:
+                case pdf_refxform_node:        /* pdf_place_form(pdf, pdf_xform_objnum(p)); */
+                case pdf_refximage_node:       /* pdf_place_image(pdf, pdf_ximage_idx(p)); */
                     /* Output a Form node in a vlist */
                     /* Output an Image node in a vlist */
                     switch (box_direction(localpos.dir)) {
@@ -1085,10 +1041,7 @@ void vlist_out(PDF pdf, halfword this_box)
                     default:
                         break;
                     }
-                    if (subtype(p) == pdf_refximage_node)       /* pdf_place_image(pdf, pdf_ximage_idx(p)); */
-                        backend_out_whatsit[pdf_refximage_node] (pdf, p);
-                    else        /* pdf_place_form(pdf, pdf_xform_objnum(p)); */
-                        backend_out_whatsit[pdf_refxform_node] (pdf, p);
+                    backend_out_whatsit[subtype(p)] (pdf, p);
                     cur.v += height(p) + depth(p);
                     break;
                 default:
@@ -1266,7 +1219,7 @@ void vlist_out(PDF pdf, halfword this_box)
         synch_pos_with_cur(pdf->posstruct, refpos, cur);
     }
     /* Finish vlist {\sl Sync\TeX} information record */
-    if (synctexoption == 1)
+    if (int_par(synctex_code))
         synctex_tsilv(this_box);
 
     if (pdf->o_mode == OMODE_DVI) {
