@@ -18,7 +18,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: ptexlib.h 2985 2009-08-11 21:53:03Z hhenkel $ */
+/* $Id: ptexlib.h 3039 2009-08-22 06:12:19Z hhenkel $ */
 
 #ifndef PTEXLIB_H
 #  define PTEXLIB_H
@@ -297,8 +297,8 @@ void vf_out_image(PDF pdf, unsigned i);
 void flush_loggable_info(void);
 
 /* lua/luastuff.c */
-void luacall(int s, int nameptr);
 void luatokencall(int p, int nameptr);
+extern void late_lua(PDF pdf, halfword p);
 
 extern void check_texconfig_init(void);
 
