@@ -41,7 +41,7 @@ void init_col(void);
 #define overfull_rule     dimen_par(overfull_rule_code)
 
 static const char _svn_version[] =
-    "$Id: align.c 2899 2009-07-21 23:03:53Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/tex/align.c $";
+    "$Id: align.c 3080 2009-10-25 22:55:06Z oneiros $ $URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/align.c $";
 
 /*
 It's sort of a miracle whenever \.{\\halign} and \.{\\valign} work, because
@@ -955,9 +955,11 @@ value is changed to zero and so is the next tabskip.
 
                 if (cur_list.mode_field == -vmode) {
                     type(q) = hlist_node;
+                    subtype(q) = 0;
                     width(q) = width(p);
                 } else {
                     type(q) = vlist_node;
+                    subtype(q) = 0;
                     height(q) = height(p);
                 }
                 glue_order(q) = glue_order(p);
@@ -1076,6 +1078,7 @@ value is changed to zero and so is the next tabskip.
                         type(r) = vlist_node;
 
                     }
+                    subtype(r) = 0;
                     shift_amount(r) = 0;
                     if (u != hold_head) {       /* append blank boxes to account for spanned nodes */
                         vlink(u) = vlink(r);
