@@ -20,8 +20,8 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: extensions.c 3024 2009-08-18 21:03:56Z hhenkel $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/tex/extensions.c $";
+    "$Id: extensions.c 3044 2009-09-13 16:28:28Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/extensions.c $";
 
 #define mode          cur_list.mode_field
 #define tail          cur_list.tail_field
@@ -197,7 +197,8 @@ void do_extension(PDF pdf)
             } else {
                 switch (cur_chr) {
                 case pdf_obj_code:
-                    check_o_mode(pdf, "\\immediate\\pdfobj", OMODE_PDF, true);
+                    check_o_mode(pdf, "\\immediate\\pdfobj", 1 << OMODE_PDF,
+                                 true);
                     do_extension(pdf);  /* scan object and set |pdf_last_obj| */
                     if (obj_data_ptr(pdf, pdf_last_obj) == 0)   /* this object has not been initialized yet */
                         pdf_error("ext1",
@@ -205,13 +206,14 @@ void do_extension(PDF pdf)
                     pdf_write_obj(pdf, pdf_last_obj);
                     break;
                 case pdf_xform_code:
-                    check_o_mode(pdf, "\\immediate\\pdfxform", OMODE_PDF, true);
+                    check_o_mode(pdf, "\\immediate\\pdfxform", 1 << OMODE_PDF,
+                                 true);
                     do_extension(pdf);  /* scan form and set |pdf_last_xform| */
                     pdf_cur_form = pdf_last_xform;
                     ship_out(pdf, obj_xform_box(pdf, pdf_last_xform), false);
                     break;
                 case pdf_ximage_code:
-                    check_o_mode(pdf, "\\immediate\\pdfximage", OMODE_PDF,
+                    check_o_mode(pdf, "\\immediate\\pdfximage", 1 << OMODE_PDF,
                                  true);
                     do_extension(pdf);  /* scan image and set |pdf_last_ximage| */
                     pdf_write_image(pdf, pdf_last_ximage);
@@ -227,29 +229,29 @@ void do_extension(PDF pdf)
         break;
     case pdf_annot_node:
         /* Implement \.{\\pdfannot} */
-        check_o_mode(pdf, "\\pdfannot", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfannot", 1 << OMODE_PDF, false);
         scan_annot(pdf);
         break;
     case pdf_catalog_code:
         /* Implement \.{\\pdfcatalog} */
-        check_o_mode(pdf, "\\pdfcatalog", OMODE_PDF, true); /* writes an object */
+        check_o_mode(pdf, "\\pdfcatalog", 1 << OMODE_PDF, true);        /* writes an object */
         scan_pdfcatalog(pdf);
         break;
     case pdf_dest_node:
         /* Implement \.{\\pdfdest} */
-        check_o_mode(pdf, "\\pdfdest", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfdest", 1 << OMODE_PDF, false);
         scan_pdfdest(pdf);
         break;
     case pdf_end_link_node:
         /* Implement \.{\\pdfendlink} */
-        check_o_mode(pdf, "\\pdfendlink", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfendlink", 1 << OMODE_PDF, false);
         if (abs(mode) == vmode)
             pdf_error("ext1", "\\pdfendlink cannot be used in vertical mode");
         new_whatsit(pdf_end_link_node);
         break;
     case pdf_end_thread_node:
         /* Implement \.{\\pdfendthread} */
-        check_o_mode(pdf, "\\pdfendthread", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfendthread", 1 << OMODE_PDF, false);
         new_whatsit(pdf_end_thread_node);
         break;
     case pdf_font_attr_code:
@@ -259,7 +261,7 @@ void do_extension(PDF pdf)
            empty \TeX{} string, and surely nobody will want to set
            \.{\\pdffontattr} to a string containing a single zero, as that
            would be nonsensical in the PDF output. */
-        check_o_mode(pdf, "\\pdffontattr", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdffontattr", 1 << OMODE_PDF, false);
         scan_font_ident();
         k = cur_val;
         if (k == null_font)
@@ -277,18 +279,18 @@ void do_extension(PDF pdf)
         break;
     case pdf_include_chars_code:
         /* Implement \.{\\pdfincludechars} */
-        check_o_mode(pdf, "\\pdfincludechars", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfincludechars", 1 << OMODE_PDF, false);
         pdf_include_chars(pdf);
         break;
     case pdf_info_code:
         /* Implement \.{\\pdfinfo} */
-        check_o_mode(pdf, "\\pdfinfo", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfinfo", 1 << OMODE_PDF, false);
         scan_pdf_ext_toks();
         pdf_info_toks = concat_tokens(pdf_info_toks, def_ref);
         break;
     case pdf_literal_node:
         /* Implement \.{\\pdfliteral} */
-        check_o_mode(pdf, "\\pdfliteral", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfliteral", 1 << OMODE_PDF, false);
         new_whatsit(pdf_literal_node);
         if (scan_keyword("direct"))
             set_pdf_literal_mode(tail, direct_always);
@@ -302,7 +304,7 @@ void do_extension(PDF pdf)
         break;
     case pdf_colorstack_node:
         /* Implement \.{\\pdfcolorstack} */
-        check_o_mode(pdf, "\\pdfcolorstack", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfcolorstack", 1 << OMODE_PDF, false);
         /* Scan and check the stack number and store in |cur_val| */
         scan_int();
         if (cur_val >= colorstackused()) {
@@ -352,54 +354,54 @@ void do_extension(PDF pdf)
         break;
     case pdf_setmatrix_node:
         /* Implement \.{\\pdfsetmatrix} */
-        check_o_mode(pdf, "\\pdfsetmatrix", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfsetmatrix", 1 << OMODE_PDF, false);
         new_whatsit(pdf_setmatrix_node);
         scan_pdf_ext_toks();
         set_pdf_setmatrix_data(tail, def_ref);
         break;
     case pdf_save_node:
         /* Implement \.{\\pdfsave} */
-        check_o_mode(pdf, "\\pdfsave", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfsave", 1 << OMODE_PDF, false);
         new_whatsit(pdf_save_node);
         break;
     case pdf_restore_node:
         /* Implement \.{\\pdfrestore} */
-        check_o_mode(pdf, "\\pdfrestore", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfrestore", 1 << OMODE_PDF, false);
         new_whatsit(pdf_restore_node);
         break;
     case pdf_map_file_code:
         /* Implement \.{\\pdfmapfile} */
-        check_o_mode(pdf, "\\pdfmapfile", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfmapfile", 1 << OMODE_PDF, false);
         scan_pdf_ext_toks();
         pdfmapfile(def_ref);
         delete_token_ref(def_ref);
         break;
     case pdf_map_line_code:
         /* Implement \.{\\pdfmapline} */
-        check_o_mode(pdf, "\\pdfmapline", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfmapline", 1 << OMODE_PDF, false);
         scan_pdf_ext_toks();
         pdfmapline(def_ref);
         delete_token_ref(def_ref);
         break;
     case pdf_names_code:
         /* Implement \.{\\pdfnames} */
-        check_o_mode(pdf, "\\pdfnames", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfnames", 1 << OMODE_PDF, false);
         scan_pdf_ext_toks();
         pdf_names_toks = concat_tokens(pdf_names_toks, def_ref);
         break;
     case pdf_obj_code:
         /* Implement \.{\\pdfobj} */
-        check_o_mode(pdf, "\\pdfobj", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfobj", 1 << OMODE_PDF, false);
         scan_obj(pdf);
         break;
     case pdf_outline_code:
         /* Implement \.{\\pdfoutline} */
-        check_o_mode(pdf, "\\pdfoutline", OMODE_PDF, true);
+        check_o_mode(pdf, "\\pdfoutline", 1 << OMODE_PDF, true);
         scan_pdfoutline(pdf);
         break;
     case pdf_refobj_node:
         /* Implement \.{\\pdfrefobj} */
-        check_o_mode(pdf, "\\pdfrefobj", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfrefobj", 1 << OMODE_PDF, false);
         scan_int();
         pdf_check_obj(pdf, obj_type_obj, cur_val);
         new_whatsit(pdf_refobj_node);
@@ -407,12 +409,12 @@ void do_extension(PDF pdf)
         break;
     case pdf_refxform_node:
         /* Implement \.{\\pdfrefxform} */
-        check_o_mode(pdf, "\\pdfrefxform", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfrefxform", 1 << OMODE_PDF, false);
         scan_pdfrefxform(pdf);
         break;
     case pdf_refximage_node:
         /* Implement \.{\\pdfrefximage} */
-        check_o_mode(pdf, "\\pdfrefximage", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfrefximage", 1 << OMODE_PDF, false);
         scan_int();
         pdf_check_obj(pdf, obj_type_ximage, cur_val);
         new_whatsit(pdf_refximage_node);
@@ -428,35 +430,35 @@ void do_extension(PDF pdf)
         break;
     case pdf_start_link_node:
         /* Implement \.{\\pdfstartlink} */
-        check_o_mode(pdf, "\\pdfstartlink", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfstartlink", 1 << OMODE_PDF, false);
         scan_startlink(pdf);
         break;
     case pdf_start_thread_node:
         /* Implement \.{\\pdfstartthread} */
-        check_o_mode(pdf, "\\pdfstartthread", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfstartthread", 1 << OMODE_PDF, false);
         new_annot_whatsit(pdf_start_thread_node);
         scan_thread_id();
         break;
     case pdf_thread_node:
         /* Implement \.{\\pdfthread} */
-        check_o_mode(pdf, "\\pdfthread", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfthread", 1 << OMODE_PDF, false);
         new_annot_whatsit(pdf_thread_node);
         scan_thread_id();
         break;
     case pdf_trailer_code:
         /* Implement \.{\\pdftrailer} */
-        check_o_mode(pdf, "\\pdftrailer", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdftrailer", 1 << OMODE_PDF, false);
         scan_pdf_ext_toks();
         pdf_trailer_toks = concat_tokens(pdf_trailer_toks, def_ref);
         break;
     case pdf_xform_code:
         /* Implement \.{\\pdfxform} */
-        check_o_mode(pdf, "\\pdfxform", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfxform", 1 << OMODE_PDF, false);
         scan_pdfxform(pdf);
         break;
     case pdf_ximage_code:
         /* Implement \.{\\pdfximage} */
-        check_o_mode(pdf, "\\pdfximage", OMODE_PDF, false);
+        check_o_mode(pdf, "\\pdfximage", 1 << OMODE_PDF, false);
         /* png, jpeg, and pdf image handling depends on this done so early: */
         fix_pdf_minorversion(pdf);
         scan_image(pdf);
@@ -510,7 +512,6 @@ void do_extension(PDF pdf)
         break;
     case late_lua_node:
         /* Implement \.{\\latelua} */
-        check_o_mode(pdf, "\\latelua", OMODE_PDF | OMODE_LUA, false);
         new_whatsit(late_lua_node);
         late_lua_name(tail) = scan_lua_state();
         (void) scan_toks(false, false);
@@ -608,8 +609,7 @@ integer pdf_last_annot;
 /* pdflastlink needs an extra global variable */
 
 integer pdf_last_link;
-integer pdf_last_x_pos;
-integer pdf_last_y_pos;
+scaledpos pdf_last_pos = { 0, 0 };
 
 /*
 To implement primitives as \.{\\pdfinfo}, \.{\\pdfcatalog} or
