@@ -21,8 +21,8 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: printing.c 2899 2009-07-21 23:03:53Z oneiros $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/tex/printing.c $";
+    "$Id: printing.c 3080 2009-10-25 22:55:06Z oneiros $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/printing.c $";
 
 #define font_id_text(A) cs_text(font_id_base+(A))
 
@@ -263,8 +263,15 @@ void print(integer s)
                 print_char(0x80 + ((s % 0x1000) % 0x40));
             } else if (s >= 0x110000) {
                 int c = s - 0x110000;
-                assert(c < 256);
-                print_char(c);
+                if (c >= 256) {
+                    pdf_warning("print", "bad raw byte to print (c=",
+                                true, false);
+                    print_int(c);
+                    tprint("), skipped.");
+                    print_ln();
+                } else {
+                    print_char(c);
+                }
             } else {
                 print_char(0xF0 + (s / 0x40000));
                 print_char(0x80 + ((s % 0x40000) / 0x1000));
