@@ -21,8 +21,8 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: scanning.c 2981 2009-08-10 15:26:10Z taco $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/tex/scanning.c $";
+    "$Id: scanning.c 3037 2009-08-20 20:15:09Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/scanning.c $";
 
 
 
@@ -450,10 +450,10 @@ static boolean short_scan_something_internal(int cmd, int chr, int level,
                     cur_val = pdf_last_annot;
                     break;
                 case pdf_last_x_pos_code:
-                    cur_val = pdf_last_x_pos;
+                    cur_val = pdf_last_pos.h;
                     break;
                 case pdf_last_y_pos_code:
-                    cur_val = pdf_last_y_pos;
+                    cur_val = pdf_last_pos.v;
                     break;
                 case pdf_retval_code:
                     cur_val = pdf_retval;
