@@ -23,8 +23,8 @@
 
 
 static const char _svn_version[] =
-    "$Id: texfileio.c 2867 2009-07-16 10:51:07Z taco $"
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/tex/texfileio.c $";
+    "$Id: texfileio.c 3032 2009-08-19 21:03:29Z hhenkel $"
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/texfileio.c $";
 
 #define end_line_char int_par(end_line_char_code)
 
@@ -721,8 +721,6 @@ We have |job_name=0| if and only if the `\.{log}' file has not been opened,
 except of course for a short time just after |job_name| has become nonzero.
 */
 
-FILE *dvi_file;                 /* the device-independent output goes here */
-str_number output_file_name;    /* full name of the output file */
 str_number texmf_log_name;      /* full name of the log file */
 
 /*
