@@ -21,8 +21,8 @@
 #include <ptexlib.h>
 
 static const char _svn_version[] =
-    "$Id: texmath.c 2944 2009-08-02 22:14:29Z oneiros $ "
-    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.43.0/source/texk/web2c/luatexdir/tex/texmath.c $";
+    "$Id: texmath.c 3080 2009-10-25 22:55:06Z oneiros $ "
+    "$URL: http://foundry.supelec.fr/svn/luatex/tags/beta-0.44.0/source/texk/web2c/luatexdir/tex/texmath.c $";
 
 #define mode          cur_list.mode_field
 #define head          cur_list.head_field
@@ -896,7 +896,16 @@ void enter_display_math(void)
     scaled s;                   /* new |display_indent| */
     pointer p;
     integer n;                  /* scope of paragraph shape specification */
-    if (head == tail) {         /* `\.{\\noindent\$\$}' or `\.{\$\${ }\$\$}' */
+    if (head == tail ||         /* `\.{\\noindent\$\$}' or `\.{\$\${ }\$\$}' */
+        (vlink(head) == tail && /* the 2nd of \.{\$\${ }\$\$} \.{\$\${ }\$\$} */
+         type(tail) == whatsit_node &&
+         subtype(tail) == local_par_node && vlink(tail) == null)) {
+        if (vlink(head) == tail) {
+            /* bug \#270: |resume_after_display| inserts a |local_par_node|, but if 
+               there is another display immediately following, we have to get rid
+               of that node */
+            flush_node(tail);
+        }
         pop_nest();
         w = -max_dimen;
     } else {
@@ -1912,7 +1921,8 @@ void resume_after_display(void)
     push_nest();
     mode = hmode;
     space_factor = 1000;
-    tail_append(make_local_par_node());
+    tail_append(make_local_par_node()); /* this needs to be intercepted in 
+                                           the display math start ! */
     get_x_token();
     if (cur_cmd != spacer_cmd)
         back_input();
