@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-/* $Id: texnodes.h 2977 2009-08-08 12:49:35Z hhenkel $ */
+/* $Id: texnodes.h 3057 2009-10-08 11:47:45Z taco $ */
 
 #include <stdarg.h>
 
@@ -175,6 +175,11 @@ typedef enum {
 #  define synctex_line_kern(a) vlink((a)+3)
 
 #  define box_node_size 9
+
+#  define HLIST_SUBTYPE_UNKNOWN 0
+#  define HLIST_SUBTYPE_LINE 1
+#  define HLIST_SUBTYPE_HBOX 2
+
 #  define width(a)            varmem[(a)+2].cint
 #  define depth(a)            varmem[(a)+3].cint
 #  define height(a)           varmem[(a)+4].cint
